@@ -5,13 +5,22 @@ from asgiref.local import Local
 from django.apps import apps
 
 
+def _is_django_module(module):
+    """Return True if the given module is nested under Django."""
+    return module.__name__.startswith('django.')
+
+
 def watch_for_translation_changes(sender, **kwargs):
     """Register file watchers for .mo files in potential locale paths."""
     from django.conf import settings
 
     if settings.USE_I18N:
         directories = [Path('locale')]
-        directories.extend(Path(config.path) / 'locale' for config in apps.get_app_configs())
+        directories.extend(
+            Path(config.path) / 'locale'
+            for config in apps.get_app_configs()
+            if not _is_django_module(config.module)
+        )
         directories.extend(Path(p) for p in settings.LOCALE_PATHS)
         for path in directories:
             sender.watch_dir(path, '**/*.mo')
